package org.openxma.addons.ui.pageanimation;


import java.util.HashMap;
import java.util.Map;

import org.eclipse.nebula.animation.AnimationRunner;
import org.eclipse.nebula.animation.effects.AbstractEffect;
import org.eclipse.nebula.animation.movement.IMovement;
import org.eclipse.nebula.animation.movement.LinearInOut;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Control;

import at.spardat.xma.page.PageClient;
import at.spardat.xma.page.effects.DefaultPageEffects;

public class AnimatedPageEffects extends DefaultPageEffects {
	
	private final static String APPLIED_CONDITION_KEY = "appliedCondition";

	AnimationRunner animationRunner;
	
	Map<Control, Boolean> collapseMap;	
	int collapseDuration = 150;
	IMovement collapseMovement;

	public AnimationRunner getAnimationRunner() {
		if (animationRunner == null) {
			animationRunner = new AnimationRunner();
		}
		return animationRunner;
	}

	public AnimatedPageEffects(PageClient page) {
		super(page);
	}
	
	protected boolean isCollapseAnimated(Control control) {
		return true;
	}
	
    @Override
    public void setCollapsed(Control control, boolean condition) {
        if (collapseMap != null && isCollapseAnimated(control)) {
            addControlToCollapse(control,condition);
        } else {
        	setCollapsedWithoutAnimation(control, condition);
            setAppliedCondition(control, condition);
        }
    }
    
    protected void setCollapsedWithoutAnimation(Control control, boolean condition) {
    	super.setCollapsed(control, condition);
    } 
  
    public void addControlToCollapse(Control control, boolean condition) {
    	Boolean appliedCondition = (Boolean) control.getData(APPLIED_CONDITION_KEY);
    	if (appliedCondition == null || appliedCondition != condition) {    	
    		collapseMap.put(control,condition);
    	}
    }  
    
    public void setAppliedCondition(Control control, boolean appliedCondition) {
    	control.setData(APPLIED_CONDITION_KEY, appliedCondition);
    }
	
	@Override
	public void initializePageEffects() {
		super.initializePageEffects();		
		collapseMap = new HashMap<Control,Boolean>();		
	}
	
	@Override
	public void applyPageEffects() {	 
		if (collapseMap != null && collapseMap.keySet().size()>0) {
			CollapseEffect effect = createCollapseEffect(collapseMap);			
			getAnimationRunner().runEffect(effect);
		} else {
			super.applyPageEffects();							
		}
		collapseMap = null;			
	}	
	    	
	protected CollapseEffect createCollapseEffect(Map<Control, Boolean> collapseMap) {
		if (collapseMovement == null) {
			collapseMovement = new LinearInOut();
		}
		CollapseEffect effect = new CollapseEffect(this, collapseDuration, collapseMovement, collapseMap);
		return effect;
	}		
	
	public int getCollapseDuration() {
		return collapseDuration;
	}

	public void setCollapseDuration(int collapseDuration) {
		this.collapseDuration = collapseDuration;
	}

	public IMovement getCollapseMovement() {
		return collapseMovement;
	}

	public void setCollapseMovement(IMovement collapseMovement) {
		this.collapseMovement = collapseMovement;
	}

	static class CollapseEffect extends AbstractEffect {    
		
		Map<Control, Boolean> collapseMap;	
		
		AnimatedPageEffects collapseHelper;    	
		public CollapseEffect(AnimatedPageEffects pageAnimation, long lengthMilli, IMovement movement, final Map<Control, Boolean> collapseMap) {
			super(lengthMilli, movement,null,null);
			this.runnableOnStop = createOnStopCollapse(collapseMap);
			this.collapseMap = collapseMap;			
			easingFunction.init(0, 100, (int) lengthMilli);			
			this.collapseHelper = pageAnimation;
		}
		
		@Override
		public void applyEffect(long currentTime) {								
			int collapsePercent = (int)easingFunction.getValue(currentTime);
			for (Control c : collapseMap.keySet()) {
				applyCollapseAnimation(c,collapseMap.get(c),collapsePercent);
			}
			collapseHelper.getPage().getComposite().layout(true,true);															
		} 
				
		public void applyCollapseAnimation(Control control, boolean condition, int collapsePercent) {
		    FormData data = (FormData)control.getLayoutData();
		    FormData uncollapsedFormData = collapseHelper.getUncollapsedFormData(control,data);		    
		    if (data.top == null && data.bottom == null) {
		    	throw new RuntimeException("Collapse direction not implemented yet for FormData "+data);
		    }
			Point p = control.computeSize(SWT.DEFAULT, SWT.DEFAULT);
			int height;
			if (!condition) {
				if (uncollapsedFormData.top != null) {
					data.top.offset = uncollapsedFormData.top.offset * collapsePercent / 100;					
				}
				if (uncollapsedFormData.bottom != null) {
					data.bottom.offset = uncollapsedFormData.bottom.offset * collapsePercent / 100;					
				}				
				height = p.y * collapsePercent / 100;
			} else {
				if (uncollapsedFormData.top != null) {
					data.top.offset = uncollapsedFormData.top.offset * (100 - collapsePercent) / 100;					
				}
				if (uncollapsedFormData.bottom != null) {
					data.bottom.offset = uncollapsedFormData.bottom.offset * (100 - collapsePercent) / 100;					
				}					
				height = p.y * (100 - collapsePercent) / 100;
			}
			
			data.height = height;
			control.setVisible(true);
		}
		
		protected Runnable createOnStopCollapse(final Map<Control, Boolean> collapseMap) {
			 return new Runnable() {
				public void run() {				
					for (Control c : collapseMap.keySet()) {
						boolean condition = collapseMap.get(c);
						collapseHelper.setCollapsedWithoutAnimation(c,condition);
						collapseHelper.setAppliedCondition(c, condition);			
					}	
					collapseHelper.getPage().getComposite().layout(true,true);					
				}
			};				
		}			
		
	}    	
}

